from gconf import client_get_default, CLIENT_PRELOAD_NONE, VALUE_STRING
from md5 import new as new_key
from common import Profile

from common import Snippet, TEXT, MATH, UNKNOWN

GCONF_ROOT = "/apps/gedit-latex-plugin"

class Settings:
	def __init__(self):
		self.client = client_get_default()
		self.client.add_dir(GCONF_ROOT, CLIENT_PRELOAD_NONE)
	
	def __getitem__(self, path):
		value = self.client.get(self.__getAbsolutePath(path))
		if not value:
			return None
		return self.__resolveType(value)
				
	def __setitem__(self, path, value):
		tp = type(value)
		p = self.__getAbsolutePath(path)
		if tp == str:
			self.client.set_string(p, value)
		elif tp == int:
			self.client.set_int(p, value)
		elif tp == bool:
			self.client.set_bool(p, value)
		elif tp == list:
			self.client.set_list(p, VALUE_STRING, [str(i) for i in value])
	
	def __getAbsolutePath(self, path):
		return "%s/%s" % (GCONF_ROOT, path)
	
	def __resolveType(self, valueObj):
		""" 
		Resolves the type of a GConf value object 
		"""
		tp = valueObj.type.value_name
		if tp == "GCONF_VALUE_STRING":
			return valueObj.get_string()
		elif tp == "GCONF_VALUE_INT":
			return valueObj.get_int()
		elif tp == "GCONF_VALUE_BOOL":
			return valueObj.get_bool()
		elif tp == "GCONF_VALUE_LIST":
			return [self.__resolveType(i) for i in valueObj.get_list()]
	
	def setdefault(self, key, default):
		""" Similar to __getitem__(), but returns <default> and assigns it to <key>
			if it doesn't exist """
		if self.client.get_without_default(self.__getAbsolutePath(key)):
			return self.__getitem__(key)
		else:
			self.__setitem__(key, default)
			return default
	
	def remove_dir_r(self, path):
		""" Unsets all children of path and then removes the path itself hoping that
		the directory is really deleted then... """
		
		for child in self.client.all_dirs(path):
			self.client.unset(child)
		
		self.client.remove_dir(path)
	
	#def unset(self, key):
	#	""" Unsets (removes) an entry 
	#		(if all entries in a directory are unset, then this directory is removed) """
	#	self.client.unset(self.__getPath(key))
	
	def getSubdirs(self, key):
		""" Returns the relative subdirectories of a path """
		path = self.__getAbsolutePath(key)
		return [d[len(path) + 1:] for d in self.client.all_dirs(self.__getAbsolutePath(key))]
	
	#def addDir(self, key):
	#	""" Adds a directory """
	#	self.client.add_dir(self.__getPath(key), CLIENT_PRELOAD_NONE)
	
	#def dirExists(self, key):
	#	""" Returns True if the directory exists, False otherwise """
	#	return self.client.dir_exists(self.__getPath(key))
	
	#def removeDir(self, key):
	#	self.client.remove_dir(self.__getPath(key))
	
	##############################
	## profile methods
	
	def monitor_profiles(self, callback):
		self.client.notify_add("%s/custom-build-profiles" % GCONF_ROOT, callback)
	
	def get_profile(self, key):
		name = self.client.get_string("%s/custom-build-profiles/%s/name" % (GCONF_ROOT, key))
		commands = self.client.get_list("%s/custom-build-profiles/%s/commands" % (GCONF_ROOT, key), VALUE_STRING)
		view_command = self.client.get_string("%s/custom-build-profiles/%s/view-command" % (GCONF_ROOT, key))
		if not view_command:
			view_command = ""
		accel = self.client.get_string("%s/custom-build-profiles/%s/accel" % (GCONF_ROOT, key))
		if not accel:
			accel = ""
		
		p = Profile(name, commands, view_command, accel)
		p.key = key
		
		#print "Name: %s\nCommands:%s\nViewCommand:%s" % (p.name, p.commands, p.view_command)
		
		return p
	
	def set_profile(self, profile):
		key = new_key(profile.name).hexdigest()
		self.client.set_string("%s/custom-build-profiles/%s/name" % (GCONF_ROOT, key), profile.name)
		self.client.set_list("%s/custom-build-profiles/%s/commands" % (GCONF_ROOT, key), VALUE_STRING, profile.commands)
		self.client.set_string("%s/custom-build-profiles/%s/view-command" % (GCONF_ROOT, key), profile.view_command)
		self.client.set_string("%s/custom-build-profiles/%s/accel" % (GCONF_ROOT, key), profile.accel)
	
	def delete_profile(self, key):
		self.client.remove_dir("%s/custom-build-profiles/%s" % (GCONF_ROOT, key))
	
	def get_profiles(self):
		profiles = []
		for path in self.client.all_dirs("%s/custom-build-profiles" % GCONF_ROOT):
			key = path.split("/")[-1]
			p = self.get_profile(key)
			p.key = key
			profiles.append(p)
		return profiles
	
	#
	# Snippets
	#
	def get_snippet(self, key):
		"""
		Find a snippet by its key
		"""
		snippet = Snippet()
		snippet.key = key
		snippet.name = self.client.get_string("%s/user-defined-macros/%s/name" % (GCONF_ROOT, key))
		snippet.definition = self.client.get_string("%s/user-defined-macros/%s/definition" % (GCONF_ROOT, key))
		snippet.packages = self.client.get_list("%s/user-defined-macros/%s/packages" % (GCONF_ROOT, key), VALUE_STRING)
		scope = self.client.get_int("%s/user-defined-macros/%s/scope" % (GCONF_ROOT, key))
		snippet.scope = {0 : UNKNOWN, 1 : TEXT, 2 : MATH, 3 : TEXT|MATH}[scope]
		
		return snippet
	
	def save_snippet(self, snippet):
		"""
		Save or update a snippet
		"""
		## generate key if needed
		if not snippet.key:
			snippet.key = new_key(snippet.name).hexdigest()
		## store properties to gconf
		self.client.set_string("%s/user-defined-macros/%s/name" % (GCONF_ROOT, snippet.key), snippet.name)
		self.client.set_string("%s/user-defined-macros/%s/definition" % (GCONF_ROOT, snippet.key), snippet.definition)
		self.client.set_list("%s/user-defined-macros/%s/packages" % (GCONF_ROOT, snippet.key), VALUE_STRING, snippet.packages)
		scope = {UNKNOWN : 0, TEXT : 1, MATH : 2, TEXT|MATH : 3}[snippet.scope]
		self.client.set_int("%s/user-defined-macros/%s/scope" % (GCONF_ROOT, snippet.key), scope)
	
	def delete_snippet(self, snippet):
		path = "%s/user-defined-macros/%s" % (GCONF_ROOT, snippet.key)
		
		self.client.recursive_unset(path, False)
		#self.client.unset(path)
		
	def list_snippets(self):
		"""
		Return a list of all snippets
		"""
		snippets = []
		for path in self.client.all_dirs("%s/user-defined-macros" % GCONF_ROOT):
			key = path.split("/")[-1]
			s = self.get_snippet(key)
			snippets.append(s)
			
		return snippets
	
	def monitor_snippets(self, callback):
		"""
		Monitor the snippets gconf directory
		"""
		self.client.notify_add("%s/user-defined-macros" % GCONF_ROOT, callback)
		
