from gtk import ComboBox, ListStore, CellRendererText

## unit constants
PIXEL, INCH, MILLIMETER, CENTIMETER, POINT, PERCENT = range(6)

UNIT_ORDER = [PIXEL, INCH, MILLIMETER, CENTIMETER, POINT]

## unit labels
UNIT_LABELS = {PIXEL : "Pixels", INCH : "Inches", MILLIMETER : "Millimeters", CENTIMETER : "Centimeters",
		POINT : "Points", PERCENT : "Percent"}
		
UNIT_SHORTS = {PIXEL : "px", INCH : "in", MILLIMETER : "mm", CENTIMETER : "cm",
		POINT : "pt", PERCENT : r"%"}

class UnitConverter:
	def __init__(self, dpi):
		self.dpi = float(dpi)
		
		self.factors = [[0, 0, 0, 0, 0], [0, 0, 0, 0, 0], [0, 0, 0, 0, 0], [0, 0, 0, 0, 0], [0, 0, 0, 0, 0]]
		
		self.factors[PIXEL][INCH] = 1.0 / self.dpi				# 1 px = 1/DPI in
		self.factors[PIXEL][MILLIMETER] = 25.4 / self.dpi		# 1 in = 25.4 mm
		self.factors[PIXEL][CENTIMETER] = 2.54 / self.dpi
		self.factors[PIXEL][POINT] = 72.0 / self.dpi			# 1 pt = 1/72 in
		
		self.factors[INCH][PIXEL] = self.dpi
		self.factors[INCH][MILLIMETER] = 25.4
		self.factors[INCH][CENTIMETER] = 2.54
		self.factors[INCH][POINT] = 72.0
		
		self.factors[MILLIMETER][PIXEL] = self.dpi / 25.4
		self.factors[MILLIMETER][INCH] = 1.0 / 25.4
		self.factors[MILLIMETER][CENTIMETER] = 0.1
		self.factors[MILLIMETER][POINT] = 72.0 / 25.4
		
		self.factors[CENTIMETER][PIXEL] = self.dpi / 2.54
		self.factors[CENTIMETER][INCH] = 1.0 / 2.54
		self.factors[CENTIMETER][MILLIMETER] = 10.0
		self.factors[CENTIMETER][POINT] = 72.0 / 2.54
		
		self.factors[POINT][PIXEL] = self.dpi / 72.0
		self.factors[POINT][INCH] = 1.0 / 72.0
		self.factors[POINT][MILLIMETER] = 25.4 / 72.0
		self.factors[POINT][CENTIMETER] = 2.54 / 72.0
		
	def convert(self, value, from_unit, to_unit):
		if from_unit == to_unit:
			return value
		else:
			return value * self.factors[from_unit][to_unit]

# TODO: check enable percent

class UnitComboBox (ComboBox):
	""" 
	Provides a special ComboBox for choosing between units. When the unit is
	changed, the values of the attached SpinButtons are converted automatically.
	
	An absolute value can be specified for each SpinButton to enable conversion
	between percent and a metric unit. 
	"""
	
	#def __init__(self, dpi=96.0, default_unit=MILLIMETER):
	def __init__(self, dpi=96.0):
		## maps the attached SpinButtons to their absolute values and units
		self.spinbuttons = {}
		self.keep_aspect_ratio = False
		self.enable_percent = False
		
		self.converter = UnitConverter(dpi)
		
		## initialize widget
		self.model = ListStore(str)
		ComboBox.__init__(self, self.model)
		
		cell = CellRendererText()
  		self.pack_start(cell, True)
  		self.add_attribute(cell, "text", 0)
  		
  		for unit in UNIT_ORDER:
  			self.model.append([UNIT_LABELS[unit]])
  		
  		self.set_unit(MILLIMETER)
  		self.current_unit = MILLIMETER
  		
  		self.connect("changed", self.__unit_changed)
		
	def get_unit(self):
		return self.current_unit
	
	def get_unit_short(self):
		return UNIT_SHORTS[self.current_unit]
	
	def set_unit(self, unit):
		self.set_active(unit)
		
	def attach_spinbutton(self, spinbutton):
		## add it to the mapping
		self.spinbuttons[spinbutton] = (0, 0)
		## listen to its changes
		spinbutton.connect("value-changed", self.__spinbutton_changed)
	
	def set_absolute_value(self, spinbutton, value, unit):
		""" 
		Stores an absolute value and its unit for a specific SpinButton 
		"""
		try:
			self.spinbuttons[spinbutton] = (value, unit)
		except KeyError:
			pass
	
	def set_keep_aspect_ratio(self, setting):
		self.keep_aspect_ratio = setting
	
	def set_enable_percent(self, setting):
		self.enable_percent = setting
		self.model.append([UNIT_LABELS[PERCENT]])
		
	def __unit_changed(self, combobox):
		## convert values of the spinbuttons
		unit_from = self.current_unit
		unit_to = self.get_active()
		
		for spinbutton in self.spinbuttons.keys():
			## convert the the value from the last unit to pixels
			value = float(spinbutton.get_value())
			
			if self.current_unit == PERCENT:
				value = value / 100.0 * self.spinbuttons[spinbutton][0]
				unit_from = self.spinbuttons[spinbutton][1]
			else:
				unit_from = self.current_unit
			
			if unit_to == PERCENT:
				abs_value = self.spinbuttons[spinbutton][0]
				abs_unit = self.spinbuttons[spinbutton][1]
				
				if abs_value > 0:
					abs_value_curr_unit = self.converter.convert(abs_value, abs_unit, self.current_unit)
				
					#abs_value_curr_unit = abs_value * self.factors[abs_unit][self.current_unit]
					n_value = value / abs_value_curr_unit * 100.0
				else:
					n_value = 0.0
			else:
				n_value = self.converter.convert(value, unit_from, unit_to)
				
			spinbutton.set_value(n_value)
			
			## disable decimal places when switching to pixels
			if unit_to == PIXEL:
				spinbutton.set_digits(0)
			else:
				spinbutton.set_digits(2)
		
		self.current_unit = unit_to
	
	def __spinbutton_changed(self, spinbutton):
		if self.keep_aspect_ratio:
			abs_a = self.spinbuttons[spinbutton][0]
			for b in self.spinbuttons.keys():
				if b is not spinbutton:
					abs_b = self.spinbuttons[b][0]
					break
			
			r = float(abs_a) / float(abs_b)
			
			b.set_value(spinbutton.get_value() / r)
			
			
		
